/**
 * 
 */
package org.eclipse.cosmos.rm.validation.internal.artifacts;

import java.util.ArrayList;
import java.util.List;

/**
 * A SMLKeyUniqueSelector represents a key or unique element in an identity constraint.
 * The element contains the xml element where the constraint is applied,
 * the selector which identifies the target element for the constraint context the fields
 * on the target element which are part of the constraint.
 * @author popescu
 *
 */
public class SMLKeyUniqueSelector {
	
	protected boolean isKeyConstraint;
	/**
	 * The name of the key or unique element
	 */
	protected String name;
	
	/**
	 * The target namespace for the container element definition
	 */
	protected String targetNamespace;

	/**
	 * The namespace for the container element definition
	 */
	protected String namespace;

	/**
	 * The xml element where the constraint should be applied
	 */
	protected String containerElement;
	
	/**
	 * The context of the identity constraint
	 */
	protected String selectorPath;
	
	/**
	 * The list of fields on the selectorPath which are part of the constraint
	 */
	List fields=new ArrayList();
	
	public SMLKeyUniqueSelector(String name, boolean isKey)
	{
		this.name = name;
		isKeyConstraint = isKey;
	}

	public SMLKeyUniqueSelector(String name)
	{
		this(name, true);		
	}
	
	/**
	 * @return the containerElement
	 */
	public String getContainerElement() {
		return containerElement;
	}

	/**
	 * @param containerElement the containerElement to set
	 */
	public void setContainerElement(String containerElement) {
		this.containerElement = containerElement;
	}

	/**
	 * @return the fields
	 */
	public List getFields() {
		return fields;
	}

	/**
	 * @param fields the fields to set
	 */
	public void setFields(List fields) {
		this.fields = fields;
	}
	
	/**
	 * 
	 * @param field
	 */
	public void addField(String field) {
		fields.add(field);
	}
	

	/**
	 * @return the selectorPath
	 */
	public String getSelectorPath() {
		return selectorPath;
	}

	/**
	 * @param selectorPath the selectorPath to set
	 */
	public void setSelectorPath(String targetElement) {
		this.selectorPath = targetElement;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	public String getTargetNamespace() {
		return targetNamespace;
	}

	public void setTargetNamespace(String targetNamespace) {
		this.targetNamespace = targetNamespace;
	}

	public String getNamespace() {
		return namespace;
	}

	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public boolean isKeyConstraint() {
		return isKeyConstraint;
	}

	public void setKeyConstraint(boolean isKeyConstraint) {
		this.isKeyConstraint = isKeyConstraint;
	}

}
