package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * This class will build a map containing all phenic documents in a DOM format. 
 * The documents will be identified by the document alias 
 * @author popescu
 *
 */
public class DocumentDOMBuilder  extends AbstractDataBuilder{

	/**
	 * The ID of this builder
	 */
	public static final String ID = SMLActivator.PLUGIN_ID + ".DocumentDOMBuilder";
	
	public static final String NO_ALIAS="no_alias";
	
	/**
	 * 
	 */
	protected static DocumentDOMBuilder instance;
	protected String xmlResource;
	protected Map phenicDocMap = new HashMap();
	protected Document mainDocument;
	protected List phenicsWithNoAlias = new ArrayList();

	private DocumentDOMBuilder() {
		super();
	}

	public Object getDataStructure() {
		
		if(mainDocument == null)
			initializeDoc();
		
		return phenicDocMap;
	}

	private void initializeDoc() {
		
		DOMParser parser = new DOMParser();
		
		try {
			InputSource source = new InputSource(new FileInputStream(xmlResource));			
			parser.parse(source);
			
			mainDocument = parser.getDocument();
			NodeList list = mainDocument.getElementsByTagName("document");
			
			for(int idx=0; idx<list.getLength(); idx++)
			{
				Node node = list.item(idx);
				
				if(node.getNamespaceURI() != null && node.getNamespaceURI().equals(ISMLConstants.SMLIF_URI))
				{
					Node parent = node.getParentNode();
					
					Node data = null;
					List tempAliasList = new ArrayList();
					if(parent.getNodeName().equals(ISMLConstants.PHENICS_ELEMENT))
					{							
						//this is an SMLIF phenic document; 
						//find all aliases and add it to the phenics map												
						NodeList children = node.getChildNodes();
						for(int i=0; i<children.getLength(); i++)
						{
							Node child = children.item(i);
							if(child.getLocalName() != null)
							{
								if(child.getLocalName().equals("data"))
								{
									data = child;
								}
								else if(child.getLocalName().equals("docinfo"))
								{
									NodeList aliases = child.getChildNodes();
									for(int j=0; j<aliases.getLength(); j++)
									{
										Node aliasList = aliases.item(j);
										if(aliasList.getLocalName() != null && aliasList.getLocalName().equals("aliases"))
										{
											NodeList achildren = aliasList.getChildNodes();
											for(int k=0; k<achildren.getLength(); k++)
											{
												Node alias = achildren.item(k);
												if(alias.getLocalName() != null && alias.getLocalName().equals("alias"))
												{
													NodeList achildren1 = alias.getChildNodes();
													for(int m=0; m<achildren1.getLength(); m++)
													{
														if(achildren1.item(m).getNodeType() == Node.TEXT_NODE)
														{
															tempAliasList.add(achildren1.item(m).getNodeValue());
														}
														
													}
													
												}
											}
										}
									}
								}
							}
						}
						if(data != null)
						{
							if(tempAliasList.size() == 0)
								phenicsWithNoAlias.add(data);
							else
							{
								for(int l=0; l<tempAliasList.size(); l++)
								{
									//System.out.println(tempAliasList.get(l));									
									phenicDocMap.put(tempAliasList.get(l), data);
								}
							}
						}
					}
				}
			}
			
			if(phenicsWithNoAlias.size() > 0)
				phenicDocMap.put(NO_ALIAS, phenicsWithNoAlias);
		}
		catch(Exception exc)
		{
			exc.printStackTrace();
		}
	}

	public Document getMainDocument() {
		
		if(mainDocument == null)
			initializeDoc();
		
		return mainDocument;
	}

	public void setXmlResource(String xmlResource) {
		this.xmlResource = xmlResource;
	}

	public static DocumentDOMBuilder getInstance() {
		
		if(instance == null)
			instance = new DocumentDOMBuilder();
		return instance;
	}
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.databuilders.IDataBuilder#getPhase()
	 */
	public byte getPhase() 
	{
		return ISMLConstants.GENICS_PHASE;
	}
	
}
