package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.SMLKeyUniqueSelector;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * This builder will build a datastructure used to validate SML key and unique extensions 
 * @author popescu
 *
 */
public class SMLKeyUniqueBuilder extends AbstractDataBuilder implements IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".SMLKeyUniqueBuilder";
	
	/*Genic phase*/
    protected Map keyElements = new HashMap();    
    protected SMLKeyUniqueSelector currentKey;
    protected String containerElement;    
    protected boolean isGenicPhase= true;
    /*used during the genic phase to prefix the element names*/
    protected String currentTargetNamespace;   
    protected String currentNamespace;
    protected boolean genicPhase = true;
        
    /*Phenic phase*/
    protected List keyDataElements = new ArrayList();
    protected String currentKeyElement = null;
    
	public Object getDataStructure() {
		return keyElements;
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
	
		if(isGenicPhase)
			buildSMLKeyDefinition(uri, localName, qName, attributes);
		else //phenic phase
			buildSMLKeyDataStructure(uri, localName, qName, attributes);
	}

	/**
	 * 
	 * @param uri
	 * @param localName
	 * @param qName
	 * @param attributes
	 */
	private void buildSMLKeyDataStructure(String uri, String localName, String qName, Attributes attributes) {

		String key = getKey(localName, uri);
		Object smlKeys = keyElements.get(key);
		/* This element has a contained SML Key defintion. Add this element to the SMLKey data
		 * structure for validation*/
		if(smlKeys != null && smlKeys instanceof List)
		{
			List keys = (List)smlKeys;
			for(int idx=0; idx<keys.size(); idx++)
			{
				keyDataElements.add(key);				
				currentKeyElement = key;
				System.out.println("element with key " + key);
			}
		}
		
		else if(currentKeyElement != null)
			System.out.println("start " + uri + ":" + localName);			
		
	}

	/**
	 * 
	 * @param uri
	 * @param localName
	 * @param qName
	 * @param attributes
	 */
	private void buildSMLKeyDefinition(String uri, String localName, String qName, Attributes attributes)
	{
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName))
		{
			currentTargetNamespace = attributes.getValue(ISMLConstants.TARGET_NAMESPACE);	
			
			//find the namespace for this schema
			for (int idx=0; idx< attributes.getLength(); idx++)
			{
				String value = attributes.getValue(idx);
				if(value != null && value.equals(currentTargetNamespace) 
						&& attributes.getLocalName(idx).indexOf(ISMLConstants.TARGET_NAMESPACE) == -1)
				{	
					currentNamespace = attributes.getQName(idx);
					int i = currentNamespace.indexOf(":");
					if(i != -1)
						currentNamespace = currentNamespace.substring(i+1);
					  System.out.println( "currentNamespace " + currentNamespace );					
					break;
				}
			}
			
		}
		if (ISMLConstants.ELEMENT_ELEMENT.equals(localName))
		{
			String name = attributes.getValue(ISMLConstants.NAME_ATTRIBUTE);			
			if(name != null && currentKey == null)
			{
				containerElement = name; //store this value in case this element definition contains a key
			}
		}		
		else if (localName.equals(ISMLConstants.SML_KEY)
				&& uri.equals(ISMLConstants.SML_URI)){
			
			String name = attributes.getValue(ISMLConstants.NAME_ATTRIBUTE);
			if (name != null){
				SMLKeyUniqueSelector key = new SMLKeyUniqueSelector(name);
				key.setContainerElement(containerElement);
				key.setTargetNamespace(currentTargetNamespace);
				key.setNamespace(currentNamespace);
				currentKey = key;
			}
		}
		else if (localName.equals(ISMLConstants.SML_SELECTOR) 
				&& uri.equals(ISMLConstants.SML_URI) && currentKey != null){
		
			String name = attributes.getValue(ISMLConstants.XPATH_ATTRIBUTE);
			if(name != null)
				currentKey.setSelectorPath(name);
				
		}
		else if (localName.equals(ISMLConstants.SML_FIELD) 
				&& uri.equals(ISMLConstants.SML_URI) && currentKey != null){
			
			String name = attributes.getValue(ISMLConstants.XPATH_ATTRIBUTE);
			if(name != null)
				currentKey.addField(name);			
		}
	}

	public void endElement (String uri, String localName, String qName)
	throws SAXException
    {
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName))
		{
			currentTargetNamespace = null;
			currentNamespace = null;
		}		
		else if (ISMLConstants.GENICS_ELEMENT.equals(localName)
				&& uri.equals(ISMLConstants.SMLIF_URI))
		{
			isGenicPhase = false;
		}		
		else if (localName.equals(ISMLConstants.SML_KEY)
				&& uri.equals(ISMLConstants.SML_URI)){
    	
			String key = getKey(currentKey.getContainerElement(), currentKey.getTargetNamespace());
			
			Object data = keyElements.get(key);
			if(data != null)
			{
				((List)data).add(currentKey);
			}
			else
			{
				List list = new ArrayList();
				list.add(currentKey);
				keyElements.put(key, list);					
			}
			
			currentKey = null;	    
		}
		else if(!isGenicPhase)
		{			
			if(currentKeyElement != null)
			{
				String key = getKey(localName, uri);
				if(key.equals(currentKeyElement))
					currentKeyElement = null;
				else
				{
					System.out.println("	end " + uri + ":" + localName);
				}
			}
			
		}
    }
	
	public byte getPhase() {
		return ISMLConstants.GENICS_PHENICS_PHASE;
	}
	
	/**
	 * 
	 * @param name
	 * @param uri
	 * @return
	 */
	protected String getKey(String name, String uri)
	{
		if(name == null || uri == null)
			return "";
		
		StringBuffer key = new StringBuffer();
		key.append(uri).append(":").append(name);
		
		return key.toString();		
	}
}
