package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TargetElementNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TargetSchemaNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.util.ParserHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class TargetInstanceBuilder extends AbstractDataBuilder implements
		IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".TargetInstanceBuilder";
    protected TargetElementNode currentTargetElement; 
    protected boolean uriElementFound = false; 	
    protected Map dataStructure;
    protected Map targetSchemaBuilderStructure;
    protected Map substitionElementStructure;
	
	public TargetInstanceBuilder(){		
		dataStructure = new HashMap();
	}

	public Object getDataStructure() {
		return dataStructure;
	}

	public void startPhenics() {
		IDataBuilder targetSchemaBuilder  = DataBuilderRegistry.instance().getDataStructureBuilder(TargetSchemaBuilder.ID);			
		targetSchemaBuilderStructure = (Map)targetSchemaBuilder.getDataStructure();				
		IDataBuilder substitionElement  = DataBuilderRegistry.instance().getDataStructureBuilder(SubstitutionBuilder.ID);
		substitionElementStructure = (Map)substitionElement.getDataStructure();
	}
	

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
//		//no target elements defined in schema
//		if (targetRequires == null) return;
		
		TargetSchemaNode schemaNode = (TargetSchemaNode)targetSchemaBuilderStructure.get(localName);
		
		//check substitution group
		if (schemaNode == null){
			Map subGroupMap = (Map)substitionElementStructure.get(uri);
			if (subGroupMap != null){
				Object subGroup = subGroupMap.get(localName);
				if (subGroup != null)
					schemaNode = (TargetSchemaNode)targetSchemaBuilderStructure.get(ParserHelper.removeNameSpace((String)subGroup));
			}
		}
		//check if the element is defined as having a target element attribute and the ref:targetRequired is true and 
		if (schemaNode != null){
			currentTargetElement = new TargetElementNode(localName,locator.getLineNumber(),locator.getColumnNumber());
			currentTargetElement.setUri(uri);
		}
		else if ((currentTargetElement != null) && (ISMLConstants.URI_ELEMENT.equals(localName))){
			uriElementFound = true;
		}
		else
			uriElementFound = false;
	}
	
	public void characters(char[] ch, int start, int len) throws SAXException {
		//extract the uri value
		if (uriElementFound){
			String target = new String(ch, start, len);
			dataStructure.put(currentTargetElement, target);
			currentTargetElement.setTargetURI(target);
			currentTargetElement = null;
		}
	}	

	public void endElement(String uri, String localName, String qName) throws SAXException {
		if ((uriElementFound) && (ISMLConstants.URI_ELEMENT.equals(localName))){
			uriElementFound = false;
		}
	}

	public byte getPhase() {
		return ISMLConstants.PHENICS_PHASE;
	}	

}
